@@ -14,6 +14,7 @@
 # limitations under the License.
 #
 
-from rayvens.api import init, Stream
+from rayvens.api import init, meta, Stream
+from rayvens.core.common import OutputEvent
 
-__all__ = ['init', 'Stream']
+__all__ = ['init', 'meta', 'Stream', 'OutputEvent']
