@@ -30,13 +30,17 @@ def _kafka_SASL_uri(topic, kafka_brokers, password):
     security_protocol = 'SASL_SSL'
     username = "token"
     conf = 'org.apache.kafka.common.security.plain.PlainLoginModule' \
-           f' required username=\"{username}\" password=\"{password}\";'
-    uri = f'kafka:{topic}?brokers={kafka_brokers}' \
-          f'&securityProtocol={security_protocol}' \
-          f'&saslMechanism=PLAIN' \
-          f'&saslJaasConfig={conf}' \
-          f'&sslKeyPassword={password}'
-    return uri
+        f' required username=\"{username}\" password=\"{password}\";'
+
+    if password == 'modeline':
+        return f'kafka:{topic}?brokers={kafka_brokers}' \
+               f'&securityProtocol={security_protocol}' \
+               f'&saslMechanism=PLAIN'
+    return f'kafka:{topic}?brokers={kafka_brokers}' \
+           f'&securityProtocol={security_protocol}' \
+           f'&saslMechanism=PLAIN' \
+           f'&saslJaasConfig={conf}' \
+           f'&sslKeyPassword={password}'
 
 
 def kafka_source(config):
@@ -110,29 +114,39 @@ def cos_source(config):
     if 'secret_access_key' not in config:
         raise TypeError(
             'Cloud object storage source requires an secret access key.')
-    if 'endpoint' not in config:
-        raise TypeError('Cloud object storage source requires an endpoint.')
     bucket_name = config['bucket_name']
     access_key_id = config['access_key_id']
-    secret_access_key = config['secret_access_key']
-    endpoint = config['endpoint']
+    secret_access_key = 'RAW('+config['secret_access_key']+')' # add RAW() to deal with special characters in key
 
-    # Ensure this is a valid, supported endpoint:
-    split_endpoint = _parse_endpoint(endpoint)
+    split_endpoint = None
+
+    if 'endpoint' in config:
+        endpoint = config['endpoint']
+
+        # Ensure this is a valid, supported endpoint:
+        split_endpoint = _parse_endpoint(endpoint)
 
     # Resolve region:
     region = 'us-east'
     if 'region' in config:
         region = config['region']
     else:
-        region = split_endpoint[1]
+        if split_endpoint:
+            region = split_endpoint[1]
 
     # Assemble URI:
-    uri = f'aws2-s3://{bucket_name}?accessKey={access_key_id}' \
-          f'&secretKey={secret_access_key}' \
-          '&overrideEndpoint=true' \
-          f'&uriEndpointOverride={endpoint}' \
-          f'&region={region}'
+    if 'endpoint' not in config:
+
+        uri = f'aws2-s3://{bucket_name}?accessKey={access_key_id}' \
+            f'&secretKey={secret_access_key}' \
+            f'&region={region}'
+    else:
+
+        uri = f'aws2-s3://{bucket_name}?accessKey={access_key_id}' \
+            f'&secretKey={secret_access_key}' \
+            '&overrideEndpoint=true' \
+            f'&uriEndpointOverride={endpoint}' \
+            f'&region={region}'
 
     # Move after read options:
     if 'move_after_read' in config:
@@ -151,25 +165,74 @@ def cos_source(config):
 
     return {'uri': uri, 'steps': []}
 
-
-def file_source(config):
+# get event content w/o any meta data
+def file_source_raw(config):
     if 'path' not in config:
         raise TypeError('File source requires a path name.')
     path = Path(config['path'])
     if path.is_dir():
         uri = f'file:{str(path)}?'
-    else:
+    elif path.is_file():
         uri = f'file:{str(path.parent)}?filename={path.name}&'
-
-    # Keep files after being processed, default is true.
-    if 'keep_files' not in config or not config['keep_files']:
-        uri += 'delete=true'
     else:
-        uri += 'delete=false'
+        raise RuntimeError('The given path may either be a directory or a file. The given path {} is neither.'.format(path))
+
+    use_amp_in_recursive = False
+
+    # Keep files after being processed. By default, files are deleted.
+    if 'keep_file' in config and config['keep_file']:
+        uri += 'noop=true'
+        use_amp_in_recursive = True
 
     # Recursive traversal of the directory, default is false.
     if 'recursive' in config and config['recursive']:
-        uri += '&recursive=true'
+        if use_amp_in_recursive:
+            uri += '&recursive=true'
+        else:
+            uri += 'recursive=true'
+
+    return {'uri': uri, 'steps': []}
+
+# get event + meta (name) in json format.
+def file_source(config):
+    if 'path' not in config:
+        raise TypeError('File source requires a path name.')
+    if all(['move_after_read' in config,'keep_file' in config and config['keep_file']]):
+        raise RuntimeError('Moving files will necessarily delete them from the source. move_after_read and keep_file=True are mutually exclusive.')
+        
+    path = Path(config['path'])
+    if path.is_dir():
+        uri = f'file:{str(path)}?'
+    elif path.is_file():
+        uri = f'file:{str(path.parent)}?filename={path.name}&'
+    else:
+        raise RuntimeError('The given path may either be a directory or a file. The given path {} is neither.'.format(path))
+
+    use_amp_in_recursive = True
+
+    # Move after reading files from directory.
+    if 'move_after_read' in config:
+        move_path = Path(config['move_after_read'])
+        if str(move_path) == str(path):
+            raise RuntimeError(f'Path {str(move_path)} and Path {str(path)} cannot be identical.')
+        if all([path.is_file(),str(move_path) == str(path.parent)]):
+            raise RuntimeError(f'Path {str(move_path)} and Path {str(path.parent)} cannot be identical.')
+        # by default, move_path will be created if it does not exist, hence, is_dir is not tested.
+        # therefore, move reauires care when using relative paths, especially when the source is a file.
+        uri += f'move={str(move_path)}'
+    else:
+        # Keep files after being processed. By default, files are deleted.
+        if 'keep_file' in config and config['keep_file']:
+            uri += 'noop=true'
+        else:
+            use_amp_in_recursive = False
+
+    # Recursive traversal of the directory, default is false.
+    if 'recursive' in config and config['recursive']:
+        if use_amp_in_recursive:
+            uri += '&recursive=true'
+        else:
+            uri += 'recursive=true'
 
     return {'uri': uri, 'steps': []}
 
@@ -266,6 +329,7 @@ sources = {
     'telegram-source': telegram_source,
     'binance-source': binance_source,
     'cloud-object-storage-source': cos_source,
+    'file-source-raw': file_source_raw,
     'file-source': file_source,
     'file-watch-source': file_watch_source,
     'generic-source': generic_source,
@@ -302,9 +366,12 @@ def construct_source(config, endpoint, inverted=False):
     # Manage Queue access methods.
     take_from_queue = 'takeFromQueue'
     add_to_queue = 'addToQueue'
-    if config['kind'] == 'file-source':
+    if config['kind'] == 'file-source-raw':
         take_from_queue = 'takeFromFileQueue'
         add_to_queue = 'addToFileQueue'
+    if config['kind'] == 'file-source':
+        take_from_queue = 'takeFromFileQueueName'
+        add_to_queue = 'addToFileQueueName'
     elif config['kind'] == 'file-watch-source':
         take_from_queue = 'takeFromFileWatchQueue'
         add_to_queue = 'addToFileWatchQueue'
@@ -337,6 +404,7 @@ def construct_source(config, endpoint, inverted=False):
                     }]
                 }
             })
+        # print(yaml.dump(spec_list))
         return spec_list
 
     # Regular integration with only one route:
@@ -357,20 +425,27 @@ def construct_source(config, endpoint, inverted=False):
             from_queue['from']['steps'].extend(remaining_steps)
 
         spec.append(from_queue)
+    # print(yaml.dump(spec))
     return spec
 
 
 def slack_sink(config):
     if 'channel' not in config:
         raise TypeError('Kind slack-sink requires a channel.')
-    if 'webhookUrl' not in config:
-        raise TypeError('Kind slack-sink requires a webhookUrl.')
     channel = config['channel']
-    webhookUrl = config['webhookUrl']
+
+    option_values = []
+    if 'webhook_url' in config:
+        webhook_url = config['webhook_url']
+        option_values.append(f"webhookUrl={webhook_url}")
+
+    options = ""
+    if len(option_values) > 0:
+        options = "?" + "&".join(option_values)
 
     final_spec = {
         'steps': [{
-            'to': f'slack:{channel}?webhookUrl={webhookUrl}',
+            'to': f'slack:{channel}{options}',
         }]
     }
 
@@ -436,26 +511,28 @@ def cos_sink(config):
     if 'secret_access_key' not in config:
         raise TypeError(
             'Cloud object storage sink requires an secret access key.')
-    if 'endpoint' not in config:
-        raise TypeError('Cloud object storage sink requires an endpoint.')
     bucket_name = config['bucket_name']
     access_key_id = config['access_key_id']
-    secret_access_key = config['secret_access_key']
-    endpoint = config['endpoint']
+    secret_access_key = 'RAW('+config['secret_access_key']+')' # add RAW() to deal with special characters in key
 
-    file_name = None
+    split_endpoint = None
+    if 'endpoint' in config:
+        endpoint = config['endpoint']
+        # Ensure this is a valid, supported endpoint:
+        split_endpoint = _parse_endpoint(endpoint)
+
+
+    file_name = "default.txt"
     if 'file_name' in config:
         file_name = config['file_name']
 
-    # Ensure this is a valid, supported endpoint:
-    split_endpoint = _parse_endpoint(endpoint)
-
     # Resolve region:
     region = 'us-east'
     if 'region' in config:
         region = config['region']
     else:
-        region = split_endpoint[1]
+        if split_endpoint:
+            region = split_endpoint[1]
 
     # If we are uploading a file either directly or by monitoring a directory,
     # multi-part must be enabled:
@@ -464,11 +541,16 @@ def cos_sink(config):
         config['upload_type'] = "multi-part"
 
     # Assemble URI:
-    uri = f'aws2-s3://{bucket_name}?accessKey={access_key_id}' \
-          f'&secretKey={secret_access_key}' \
-          '&overrideEndpoint=true' \
-          f'&uriEndpointOverride={endpoint}' \
-          f'&region={region}'
+    if 'endpoint' not in config:
+        uri = f'aws2-s3://{bucket_name}?accessKey={access_key_id}' \
+            f'&secretKey={secret_access_key}' \
+            f'&region={region}'
+    else:
+        uri = f'aws2-s3://{bucket_name}?accessKey={access_key_id}' \
+            f'&secretKey={secret_access_key}' \
+            '&overrideEndpoint=true' \
+            f'&uriEndpointOverride={endpoint}' \
+            f'&region={region}'
 
     # Final result is a list of spec and uri pairs: [(spec, uri)]
     spec_list = []
@@ -566,22 +648,43 @@ def cos_sink(config):
 
     # This is the default behavior when user application data is
     # written into a COS file directly.
-    if 'file_name' not in config:
-        raise TypeError('Created cloud object name is required.')
-    regular_spec = {
-        'steps': [{
-            'set-header': {
-                'name': 'CamelAwsS3Key',
-                'simple': f"{file_name}"
-            }
-        }, {
-            'to': uri
-        }]
-    }
+    if 'file_name' in config:
+        regular_spec = {
+            'steps': [{
+                'set-header': {
+                    'name': 'CamelAwsS3Key',
+                    'simple': f"{file_name}"
+                }
+            }, {
+                'to': uri
+            }]
+        }
+    else:
+        regular_spec = {'steps': [{'to': uri}]}
     spec_list.append((regular_spec, None))
     return spec_list
 
 
+def file_sink(config):
+    if 'path' not in config:
+        raise TypeError('File source requires a path name.')
+    path = Path(config['path'])
+    if path.is_dir():
+        uri = f'file:{str(path)}?'
+    elif path.is_file():
+        uri = f'file:{str(path.parent)}?filename={path.name}' # TODO: test this option
+    else:
+        raise RuntimeError('Path {} is neither a directory nor a file.'.format(config['path']))
+
+    final_spec = {
+        'steps': [{
+            'to': uri,
+        }]
+    }
+
+    return [(final_spec, None)]
+
+
 def test_sink(config):
     return [({'steps': [{'log': {'message': "\"${body}\""}}]}, None)]
 
@@ -605,6 +708,7 @@ sinks = {
     'kafka-sink': kafka_sink,
     'telegram-sink': telegram_sink,
     'cloud-object-storage-sink': cos_sink,
+    'file-sink': file_sink,
     'generic-sink': generic_sink,
     'test-sink': test_sink
 }
@@ -651,6 +755,7 @@ sink_input_restriction = {
     'kafka-sink': no_restriction,
     'telegram-sink': no_restriction,
     'cloud-object-storage-sink': cos_sink_restriction,
+    'file-sink': no_restriction,
     'generic-sink': no_restriction,
     'test-sink': no_restriction
 }
